package org.qatools.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;


public class ListUtil extends QAToolsUtil {
    private static long cUniqueLong = System.currentTimeMillis();
    private static String DAFAULT_DELIM = ",";
    
    
	// -------------------------------------------------------------
	// ----- isListNullOrEmpty -------------------------------------
	// -------------------------------------------------------------

    public static <T extends Object> boolean isListNullOrEmpty(List<T> list) {
    	if (list==null || list.size()==0) {
    		return true;
    	}
    	return false;
    }

	// -------------------------------------------------------------
	// ----- isArrayNullOrEmpty ------------------------------------
	// -------------------------------------------------------------

    public static <T extends Object> boolean isArrayNullOrEmpty(T[] arr) {
    	if (arr==null || arr.length==0) {
    		return true;
    	}
    	return false;
    }

    // --------------------------------------------------------------------
    // ----- getDelimitedStr
    // --------------------------------------------------------------------

    /**
     * 
     * @param list
     * @return
     */
    public static <T extends Object> String getDelimitedStr(List<T> list, String delim) {
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	
    	if (delim==null || delim.length()==0) {
    		delim = DAFAULT_DELIM;
    	}
    	
    	String s = "";
    	String d = "";
    	for (T t : list) {
    		s += d + t.toString();
    		d = delim;
    	}
    	return s;
    }
    
    /**
     * 
     * @param list
     * @param delim
     * @return
     */
    public static <T extends Object> String getCommaDelimitedStr(List<T> list) {
    	return getDelimitedStr(list, ",");
    }

    /**
     * 
     * @param list
     * @param delim
     * @return
     */
    public static <T extends Object> String getPipeDelimitedStr(List<T> list) {
    	return getDelimitedStr(list, "|");
    }

    /**
     * 
     * @param list
     * @param delim
     * @return
     */
    public static <T extends Object> String getSpaceDelimitedStr(List<T> list) {
    	return getDelimitedStr(list, " ");
    }

    // --------------------------------------------------------------------
    // ----- 
    // --------------------------------------------------------------------

    /**
     * Retunrs a quoted comma delimited string
     * 	{1 2 3} returns '1','2','3'
     * 
     * Good for query generations
     * select foo from bar where id in (ListUtil.getQuotedCommaDelimitedStr())
     * 
     * @param list
     * @return
     */
    public static <T extends Object> String getQuotedCommaDelimitedStr(List<T> list) {
    	String s = getDelimitedStr(list,"','");
    	if (s==null || s.length()==0) {
    		return null;
    	}
    	return "'" + s + "'";
    }

    // --------------------------------------------------------------------
    // ----- 
    // --------------------------------------------------------------------

    /**
     * 
     * @param list
     * @return
     */
    public static <T extends Object> List<T> removeDups(List<T> list) {
        if (list==null || list.size()==0) {
        	return null;
        }
        
        // remove duplicates
        HashSet<T> hashSet = new HashSet<T>(list);

        // unique list
        List<T> list2 = new LinkedList<T>(hashSet);
 
        if (list2==null || list2.size()==0) {
        	return null;
        }
 
       	return list2;
    }


	// -------------------------------------------------------------
	// ----- getListFromDelimitedStr 
	// -------------------------------------------------------------

    public static List<String> getListFromPipeDelimitedStr(String str) {
    	return getListFromDelimitedStr(str,"\\|");
    }
    public static List<String> getListFromCommaDelimitedStr(String str) {
    	return getListFromDelimitedStr(str,",");
    }
    public static List<String> getListFromSemicolonDelimitedStr(String str) {
    	return getListFromDelimitedStr(str,";");
    }
    public static List<String> getListFromColonDelimitedStr(String str) {
    	return getListFromDelimitedStr(str,":");
    }
    public static List<String> getListFromDotDelimitedStr(String str) {
    	return getListFromDelimitedStr(str,"\\.");
    }
    
    public static List<String> getListFromDelimitedStr(String str, String delim) {
    	if (StringUtil.isStrNullOrEmpty(str) || StringUtil.isStrNullOrEmpty(delim)) {
    		return null;
    	}
    	
    	String[] lines = smartSplit(str,delim);    	
    	if (lines==null || lines.length==0) {
    		return null;
    	}
    	
    	List<String> mResults = new ArrayList<String>();
    	for (int i=0; i<lines.length; i++) {
    		mResults.add(lines[i]);
    	}
    	return mResults;
    }
    
	// -------------------------------------------------------------
	// ----- smartSplit 
	// -------------------------------------------------------------

    private static String[] smartSplit(String str, String delim) {
    	if (StringUtil.isStrNullOrEmpty(str) || StringUtil.isStrNullOrEmpty(delim)) {
    		return null;
    	}
    	return str.split(delim);	//TODO - make this smarter, like multi-char delims, special chars etc
    }

	// -------------------------------------------------------------
	// ----- trim 
	// -------------------------------------------------------------

    /**
     * removes leading & trailing nulls/blanks
     * 
     */
    public static List<String> trim(List<String> list) {
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	
    	List<String> newlist = new ArrayList<String>(list.size());
    	Collections.copy(newlist, list);
    	
    	do {
    		String s = newlist.get(0);
    		if (s==null || s.length()==0) {
    			newlist.remove(0);
    		} else {
    			break;
    		} 			
    	} while (true);
    	if (newlist==null || newlist.size()==0) {
    		return null;
    	}
    	
    	do {
    		int i = newlist.size() - 1;
    		String s = newlist.get(i);
    		if (s==null || s.length()==0) {
    			newlist.remove(i);
    		} else {
    			break;
    		} 			
    	} while (true);
    	if (newlist==null || newlist.size()==0) {
    		return null;
    	}
    	
    	return newlist;
    }

	// -------------------------------------------------------------
	// ----- head --------------------------------------------------
	// -------------------------------------------------------------

    /**
    *
    */
   public static <T extends Object> List<T> head(List<T> list) {
	   return head(list,10);
   }

	   
    /**
    *
    */
   public static <T extends Object> List<T> head(List<T> list, int num) {
           int size=0;

           //figure out size of new list
           if (list!=null && 0<list.size()) {
                   size = list.size();
           }
           if (num < size) {
                   size = num;
           }

           // check size
           if (size==0) {
                   return null;
           }

           //
           List<T> newList = new ArrayList<T>();
           for (int i=0; i<size; i++) {
                   newList.add(list.get(i));
           }

           return newList;
   }

	// -------------------------------------------------------------
	// ----- tail --------------------------------------------------
	// -------------------------------------------------------------

   /**
   *
   */
  public static <T extends Object> List<T> tail(List<T> list) {
	   return tail(list,10);
  }

   /**
    *
    */
   public static <T extends Object> List<T> tail(List<T> list, int num) {
           if (isListNullOrEmpty(list) || num==0) {
                   return null;
           }

           int mListSize = list.size();
           if (num <= mListSize) {
        	   return list;
           }
           
           List<T> mNewList = new ArrayList<T>();
           int mStartIdx = mListSize - num;
           for (int i=mStartIdx; i<mListSize; i++) {
        	   mNewList.add(list.get(i));
           }
           return mNewList;
   }

   
   

   
    
}